from uuid import UUID

from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, SubscriptionLifecycle
from orchestrator.workflow import StepList, done, init, step, workflow
from orchestrator.workflows.steps import resync, set_status, store_process_subscription

from gso.products.product_blocks.site import SiteTier
from gso.products.product_types.site import SiteInactive
from gso.products import ProductType
from gso.services import subscriptions
from gso.services.crm import get_customer_by_name
from gso.workflows.site.create_site import initialize_subscription


@step("Create subscription")
def create_subscription(customer: str) -> State:
    customer_id = get_customer_by_name(customer)["id"]
    product_id: UUID = subscriptions.get_product_id_by_name(ProductType.SITE)
    subscription = SiteInactive.from_product_id(product_id, customer_id)

    return {
        "subscription": subscription,
        "subscription_id": subscription.subscription_id,
    }


def generate_initial_input_form() -> FormGenerator:
    class ImportSite(FormPage):
        class Config:
            title = "Import Site"

        site_name: str
        site_city: str
        site_country: str
        site_country_code: str
        site_latitude: float
        site_longitude: float
        site_bgp_community_id: int
        site_internal_id: int
        site_tier: SiteTier
        site_ts_address: str
        customer: str

    user_input = yield ImportSite
    return user_input.dict()


@workflow(
    "Import Site",
    target=Target.SYSTEM,
    initial_input_form=generate_initial_input_form,
)
def import_site() -> StepList:
    """Workflow to import a site without provisioning it."""

    return (
        init
        >> create_subscription
        >> store_process_subscription(Target.CREATE)
        >> initialize_subscription
        >> set_status(SubscriptionLifecycle.ACTIVE)
        >> resync
        >> done
    )
